from asyncio.log import logger
from typing import List
import sys
from netmiko import ConnectHandler, BaseConnection, SSHDetect
from pandas import read_csv, DataFrame
from config import host_file, command_file, log_file, output_dir, log_level, log_fomartter
import logging


def run_commands(connection: BaseConnection, df_command: DataFrame):    
    for index, row in df_command.iterrows():
        command = str(row['command']).strip()    
        expect_string = str(row['expect_string'])
        if(str(row['command']).strip()!=""):
            if(command.strip()=="#enable_mode"):
                logger.info("Entering enable mode")
                connection.enable()
            elif(command.strip()=="#end_enable_mode"):
                logger.info("Exiting enable mode")
                connection.exit_enable_mode()
            elif(command.strip()=="#config_mode"):
                logger.info("Entering config mode")               
                connection.config_mode()
            elif(command.strip()=="#end_config_mode"):
                logger.info("Exiting config mode")
                connection.exit_config_mode()
            else:
                output= connection.send_command(command, strip_prompt=False, strip_command=False, expect_string=expect_string)
                logger.info(output)


logger = logging.getLogger(__name__)
logger.setLevel(log_level)
fomartter = logging.Formatter(log_fomartter)

stream = logging.StreamHandler()
stream.setFormatter(fomartter)
logger.addHandler(stream)

file = logging.FileHandler(log_file)
file.setFormatter(fomartter)
logger.addHandler(file)

logger.info("Start !")

df_host = read_csv(host_file)   
df_command = read_csv(command_file)
with open(log_file, 'a') as file_output:  
    #file_output.truncate()
    sys.stdout = file_output

    for index, row in df_host.iterrows():    
        site_name = str(row['name']) 
        logger.info("Connect to site " + site_name + " with host " + str(row['host']))
        if(site_name==''):
            row["session_log"]=output_dir+str(index)+'.txt'
        else:
            row["session_log"]=output_dir+site_name+'.txt'
        row = row.drop(labels=['name']) 
        if(str(row["device_type"]).strip()=="autodetect"):
            guesser = SSHDetect(**row)
            best_match = guesser.autodetect()   
            if(best_match!=''):    
                row["device_type"] = best_match
        try:
            with ConnectHandler(**row) as device_connect:
                logger.info("Connect to host : " + str(row['host']) + " success !")
                run_commands(device_connect, df_command)                
        except Exception as ex:
            logger.error("ERROR when execute command in site " + site_name + " with device "+ str(row["host"]))
            logger.error(ex)

    sys.stdout.close()

logger.info("End !")
